
# q-expansion of newform 448.4.f.b, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 448
weight = 4
poly_data = [9, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 8, 0, -2], 3], [[-3, 0, 6, 0], 1], [[0, 4, 0, 2], 1]]

hecke_ring_character_values = [[127, [-1, 0, 0, 0]], [197, [1, 0, 0, 0]], [129, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, -1, 0, 0], [0, 0, 0, -1], [0, -1, 1, 0], [0, 0, -2, 0], [0, 0, 0, 1], [0, 0, 0, -8], [0, -25, 0, 0], [0, 0, 6, 0], [-126, 0, 0, 0], [0, 20, 0, 0], [-70, 0, 0, 0], [0, 0, 0, -6], [0, 0, 22, 0], [0, 72, 0, 0], [450, 0, 0, 0], [0, 45, 0, 0], [0, 0, 0, 57], [0, 0, 30, 0], [0, 0, 34, 0], [0, 0, 0, 78], [0, 0, 34, 0], [0, -189, 0, 0], [0, 0, 0, 34], [0, 0, 0, -76], [0, 0, 0, 43], [0, -214, 0, 0], [0, 0, -74, 0], [1834, 0, 0, 0], [2070, 0, 0, 0], [0, 0, -34, 0], [0, 135, 0, 0], [-630, 0, 0, 0], [0, -335, 0, 0], [-1566, 0, 0, 0], [0, 0, -186, 0], [0, 0, 0, 13], [0, 0, 134, 0], [0, -558, 0, 0], [0, 0, 0, -15], [0, 0, -106, 0], [0, 0, 0, 251], [0, 0, 82, 0], [1510, 0, 0, 0], [810, 0, 0, 0], [0, -110, 0, 0], [0, 0, -34, 0], [0, -776, 0, 0], [0, -117, 0, 0], [0, 0, 0, 139], [-630, 0, 0, 0], [0, 0, 350, 0], [0, 0, 0, 272], [0, -135, 0, 0], [0, 0, 0, -216], [0, 0, -142, 0], [0, 0, 0, 225], [0, 380, 0, 0], [1370, 0, 0, 0], [-6678, 0, 0, 0], [0, 929, 0, 0], [0, 0, 0, 267], [0, 587, 0, 0], [0, -2070, 0, 0], [0, 0, 0, -202], [-6750, 0, 0, 0], [0, 0, 158, 0], [-3850, 0, 0, 0], [0, 0, 422, 0], [0, 0, 0, 41], [0, 0, 0, -664], [0, 0, 214, 0], [0, -92, 0, 0], [2410, 0, 0, 0], [0, 0, -562, 0], [0, -684, 0, 0], [1386, 0, 0, 0], [0, 0, 0, 197], [-378, 0, 0, 0], [0, 0, 0, 806], [0, 1215, 0, 0], [2882, 0, 0, 0], [0, 0, 206, 0], [0, 0, 0, 776], [0, 2770, 0, 0], [0, 0, 734, 0], [-4734, 0, 0, 0], [-7450, 0, 0, 0], [0, 0, 0, -883], [0, 0, -46, 0], [0, 1143, 0, 0], [0, 900, 0, 0], [0, 0, -74, 0], [0, 0, 870, 0], [0, 0, -714, 0], [0, -2286, 0, 0], [0, 0, 0, 785], [0, 0, 0, 386], [0, -2351, 0, 0], [-7558, 0, 0, 0], [0, 0, 198, 0], [10890, 0, 0, 0], [0, -1089, 0, 0], [18774, 0, 0, 0], [0, 0, 118, 0], [0, 0, 0, -1800], [0, 4617, 0, 0], [0, 0, 0, 692], [0, 0, 530, 0], [0, 0, 0, 554], [0, -1928, 0, 0], [16450, 0, 0, 0], [630, 0, 0, 0], [0, -875, 0, 0], [0, 0, 194, 0], [-8442, 0, 0, 0], [0, -2581, 0, 0], [0, 1602, 0, 0], [-30870, 0, 0, 0], [0, 0, -1602, 0], [0, 0, 0, 1959], [31570, 0, 0, 0], [0, 0, 0, 1231], [0, 0, -282, 0], [0, 1915, 0, 0], [10458, 0, 0, 0], [13426, 0, 0, 0], [0, -3600, 0, 0], [0, -4282, 0, 0], [0, 0, 0, -743], [0, 0, -458, 0], [0, 0, 950, 0], [0, 0, -962, 0], [-28910, 0, 0, 0], [0, 0, 0, -1618], [0, 0, 0, 2372], [0, 0, 0, -897], [0, -4457, 0, 0], [0, 0, 0, 645], [2214, 0, 0, 0], [0, -6995, 0, 0], [-20358, 0, 0, 0], [0, 0, -350, 0], [0, 0, 1070, 0], [0, 0, 0, -167], [0, -990, 0, 0], [0, 0, 0, -729], [0, 0, 0, -2514], [0, 4385, 0, 0], [0, 0, 1074, 0], [-3710, 0, 0, 0], [0, 0, 0, -1492], [0, 0, -1474, 0], [0, 6462, 0, 0], [0, 0, 1102, 0], [0, 0, 1950, 0], [0, 0, 978, 0], [0, 0, 0, 2204], [0, 0, 0, -146], [0, 0, 0, 505], [0, 0, -2834, 0], [-42390, 0, 0, 0], [0, 0, 1206, 0], [0, -6615, 0, 0], [-8190, 0, 0, 0], [0, 10566, 0, 0], [0, 0, 2658, 0], [0, 0, 0, 1119]]
